@@ -171,6 +171,8 @@ on_screen_changed (GnomeRRScreen *scr,
   }
 
   app->labeler = gnome_rr_labeler_new (app->current_configuration);
+  if (gtk_widget_has_focus (app->panel))
+     gnome_rr_labeler_show (app->labeler);
 
   select_current_output_from_dialog_position (app);
 }
@@ -325,7 +327,7 @@ rebuild_rotation_combo (App *app)
   } RotationInfo;
   static const RotationInfo rotations[] = {
     { GNOME_RR_ROTATION_0, N_("Normal") },
-    { GNOME_RR_ROTATION_90, N_("Anti-Clockwise") },
+    { GNOME_RR_ROTATION_90, N_("Counterclockwise") },
     { GNOME_RR_ROTATION_270, N_("Clockwise") },
     { GNOME_RR_ROTATION_180, N_("180 Degrees") },
   };
@@ -2373,6 +2375,26 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
 			       win_rect.y + win_rect.height / 2);
 }
 
+static gboolean
+dialog_focus_in_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+  App *app = data;
+
+  if (app->labeler)
+    gnome_rr_labeler_show (app->labeler);
+  return FALSE;
+}
+
+static gboolean
+dialog_focus_out_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+  App *app = data;
+
+  if (app->labeler)
+    gnome_rr_labeler_hide (app->labeler);
+  return FALSE;
+}
+
 static void
 on_toplevel_realized (GtkWidget *widget,
                       App       *app)
@@ -2476,6 +2498,10 @@ run_application (void)
   app->clock_settings = g_settings_new (CLOCK_SCHEMA);
 
   app->panel = _gtk_builder_get_widget (builder, "display-panel");
+  g_signal_connect_after (app->panel, "focus-in-event",
+                          G_CALLBACK (dialog_focus_in_cb), app);
+  g_signal_connect_after (app->panel, "focus-out-event",
+                          G_CALLBACK (dialog_focus_out_cb), app);
 
   if (!app->panel)
     g_warning ("Missing display-panel object");
