@@ -60,6 +60,7 @@ struct _GnomeControlCenterPrivate
   GtkWidget  *scrolled_window;
   GtkWidget  *window;
   GtkWidget  *search_entry;
+  GtkWidget  *lock_button;
 
   GMenuTree  *menu_tree;
   GtkListStore *store;
@@ -129,6 +130,9 @@ activate_panel (GnomeControlCenter *shell,
           /* create the panel plugin */
           panel = g_object_new (panel_type, "shell", shell, NULL);
 
+          gtk_lock_button_set_permission (GTK_LOCK_BUTTON (priv->lock_button),
+                                          cc_panel_get_permission (CC_PANEL (panel)));
+
           box = gtk_alignment_new (0, 0, 1, 1);
           gtk_alignment_set_padding (GTK_ALIGNMENT (box), 6, 6, 6, 6);
 
@@ -161,6 +165,10 @@ activate_panel (GnomeControlCenter *shell,
                              nat_height);
           return;
         }
+      else
+        {
+          g_warning ("Could not find the loadable module for panel '%s'", id);
+        }
     }
 }
 
@@ -176,6 +184,8 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
   priv->filter_string = g_strdup ("");
   gtk_entry_set_text (GTK_ENTRY (priv->search_entry), "");
 
+  gtk_lock_button_set_permission (GTK_LOCK_BUTTON (priv->lock_button), NULL);
+
   /* reset window title and icon */
   gtk_window_set_title (GTK_WINDOW (priv->window), priv->default_window_title);
   gtk_window_set_default_icon_name (priv->default_window_icon);
@@ -430,13 +440,23 @@ static void
 search_entry_changed_cb (GtkEntry                  *entry,
                          GnomeControlCenterPrivate *priv)
 {
+  char *str;
 
   /* if the entry text was set manually (not by the user) */
   if (!g_strcmp0 (priv->filter_string, gtk_entry_get_text (entry)))
     return;
 
+  /* Don't re-filter for added trailing or leading spaces */
+  str = g_strdup (gtk_entry_get_text (entry));
+  g_strstrip (str);
+  if (!g_strcmp0 (str, priv->filter_string))
+    {
+      g_free (str);
+      return;
+    }
+
   g_free (priv->filter_string);
-  priv->filter_string = g_strdup (gtk_entry_get_text (entry));
+  priv->filter_string = str;
 
   g_object_set (priv->search_renderer,
                 "search-string", priv->filter_string,
@@ -528,7 +548,6 @@ setup_search (GnomeControlCenter *shell)
   search_scrolled = W (priv->builder, "search-scrolled-window");
   gtk_container_add (GTK_CONTAINER (search_scrolled), search_view);
 
-
   /* add the custom renderer */
   priv->search_renderer = (GtkCellRenderer*) shell_search_renderer_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (search_view),
@@ -547,6 +566,7 @@ setup_search (GnomeControlCenter *shell)
   /* setup the search entry widget */
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "search-entry");
   priv->search_entry = widget;
+  priv->filter_string = g_strdup ("");
 
   g_signal_connect (widget, "changed", G_CALLBACK (search_entry_changed_cb),
                     priv);
@@ -557,6 +577,14 @@ setup_search (GnomeControlCenter *shell)
                     priv);
 }
 
+static void
+setup_lock (GnomeControlCenter *shell)
+{
+  GnomeControlCenterPrivate *priv = shell->priv;
+
+  priv->lock_button = W (priv->builder, "lock-button");
+}
+
 static void
 maybe_add_category_view (GnomeControlCenter *shell,
                          const char         *name)
@@ -713,6 +741,7 @@ notebook_switch_page_cb (GtkNotebook               *book,
     {
       gtk_widget_hide (W (priv->builder, "home-button"));
       gtk_widget_show (W (priv->builder, "search-entry"));
+      gtk_widget_hide (W (priv->builder, "lock-button"));
     }
   else
     {
@@ -1067,6 +1096,8 @@ gnome_control_center_init (GnomeControlCenter *self)
   /* setup search functionality */
   setup_search (self);
 
+  setup_lock (self);
+
   /* store default window title and name */
   priv->default_window_title = g_strdup (gtk_window_get_title (GTK_WINDOW (priv->window)));
   priv->default_window_icon = g_strdup (gtk_window_get_icon_name (GTK_WINDOW (priv->window)));
